/*
 * Copyright (c) 2000, 2001, 2002, 2003, 2004, 2005, 2008, 2009
 *	The President and Fellows of Harvard College.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE UNIVERSITY AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE UNIVERSITY OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * consoletest.c
 *
 * 	Tests whether console can be written to.
 *
 * This should run correctly when open and write syscalls are correctly implemented
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <err.h>
#include <test161/test161.h>

int
main(int argc, char **argv)
{

	// Assume argument passing is *not* supported.

	(void) argc;
	(void) argv;

	int fd, fd1;
	// Attempt to open a file that we 'know' exists
	fd = open("bin/true", O_RDONLY);
	if(fd < 0) {
		err(-1, "Open syscall failed");
	}
	else if(fd < 3) {
		warnx("Open syscall returned number used by standard file descriptors (0,1,2)");
	}

	// Attempt to open the same file again. We should get a different fd
	fd1 = open("bin/true", O_RDONLY);
	if(fd1 < 0) {
		err(-1, "Open syscall failed");
	}
	else if(fd1 < 3) {
		warnx("Open syscall returned number used by standard file descriptors (0,1,2)");
	}
	else if(fd1 == fd) {
		err(-1, "Open syscall returned same file descriptor for second open() call\n");
	}


	success(TEST161_SUCCESS, SECRET, "/testbin/opentest");
	return 0;
}
